from janome.tokenizer import Tokenizer
import re
from japana.dictionary import lookup_dic
from japana.kanji_count import extract_unicode_block
from japana.progress_bar import printProgressBar
from japana.word_classifier import word_classifier


def word_count(text, kana=False, asc=True, _dict=False, ignore=True, celery=None):
    kanji = r'[ぁ-ゟ]*[㐀-䶵一-鿋豈-頻][ぁ-ゟ]*'
    letters = r'[ぁ-んァ-ン一-龯]|[㐀-䶵一-鿋豈-頻]\s'
    t = Tokenizer()
    tokens = t.tokenize(''.join(extract_unicode_block(letters, text)))
    print("total word count: ", len(tokens))

    words = dict()
    for token in tokens:
        word = dict()
        if words.get(token.base_form) is None and re.match(letters if kana else kanji, token.base_form):
            word["word"] = token.base_form
            word["frequency"] = 1
            words[token.base_form] = word
        elif words.get(token.base_form):
            words[token.base_form]["frequency"] += 1

    print("unique word count: ", len(words))

    words = [v for v in words.values()]
    words = sorted(words, key=lambda k: k['frequency'], reverse=not asc)
    word_list = list()
    for index, value in enumerate(words, start=1):
        word_dic = dict()
        if celery:
            meta = {'current': index, 'total': len(words)}
            celery.update_state(state='PROGRESS', meta=meta)
        printProgressBar(index, len(words), suffix='words has been done', length=50)
        if _dict:
            try:
                word_dic = lookup_dic(value["word"], ignore)
                if word_dic is None:
                    pass
                word_dic['frequency'] = value["frequency"]
                word_dic['jlpt'] = word_classifier(word_dic['word'])
                word_list.append(word_dic)

            except:
                pass
        else:
            word_dic['word'] = value["word"]
            word_dic['jlpt'] = word_classifier(word_dic['word'])
            word_dic['frequency'] = value["frequency"]
            word_list.append(word_dic)
    return word_list





from jamdict import Jamdict, config
jmd = Jamdict(db_file=config.get_file('JAMDICT_DB'))


def lookup_dic(word, igonre=True):
    word_dic = dict()
    result = jmd.lookup(word, strict_lookup=True, lookup_chars=False)
    result = result.to_json()['entries']
    for entry in result:
        if entry['kanji'][0]['text'] == word:
            word_dic['word'] = entry['kanji'][0]['text']
            word_dic['pronunciation'] = ""
            for k, p in enumerate(entry['kana'], start=1):
                if len(entry['kana']) == 1:
                    word_dic['pronunciation'] += p['text']
                else:
                    word_dic['pronunciation'] += str(k) + ". " + p['text'] + "<br>"
            word_dic['meaning'] = ""
            for j, meaning in enumerate(entry['senses'], start=1):
                if j >= 3:
                    break
                if len(entry['senses']) > 1:
                    word_dic['meaning'] += str(j) + ". "
                for i, one_sense in enumerate(meaning['SenseGloss'], start=1):
                    if i >= 5:
                        break
                    word_dic['meaning'] += one_sense['text']
                    if i != len(meaning['SenseGloss']) and i != 4:
                        word_dic['meaning'] += "; "
                if j != len(entry['senses']) and j != 2:
                    word_dic['meaning'] += "<br>"
            return word_dic
    if not igonre:
        return {"word": word}
    return None




import re
from collections import OrderedDict

hiragana_full = r'[ぁ-ゟ]'
katakana_full = r'[゠-ヿ]'
kanji = r'[㐀-䶵一-鿋豈-頻]'
radicals = r'[⺀-⿕]'
katakana_half_width = r'[｟-ﾟ]'
alphanum_full = r'[！-～]'
symbols_punct = r'[、-〿]'
misc_symbols = r'[ㇰ-ㇿ㈠-㉃㊀-㋾㌀-㍿]'
ascii_char = r'[ -~]'


def extract_unicode_block(unicode_block, string):
    ''' extracts and returns all texts from a unicode block from string argument.
        Note that you must use the unicode blocks defined above, or patterns of similar form '''
    return re.findall( unicode_block, string)


def remove_unicode_block(unicode_block, string):
    ''' removes all chaacters from a unicode block and returns all remaining texts from string argument.
        Note that you must use the unicode blocks defined above, or patterns of similar form '''
    return re.sub(unicode_block, '', string)


def remove_duplicate(str1):
    return "".join(OrderedDict.fromkeys(str1))


def list_kanji(str):
    f = open(str, "r")
    vocab = f.read()
    f.close()
    vocab = ''.join(extract_unicode_block(kanji, vocab))
    kanji_list = remove_duplicate(vocab)
    print("unique kanji count: ", len(kanji_list))
    return kanji_list
